package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysFileService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.platform.bean.SysFile;
import cn.estsh.i3plus.pojo.platform.repository.SysFileRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 文件资源服务接口
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 16:28
 * @Modify :
 **/
@Service
public class SysFileServiceImpl implements ISysFileService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysFileServiceImpl.class);

    @Autowired
    public SysFileRepository sysFileRDao;

    @Override
    @ApiOperation(value = "新增资源信息")
    public SysFile insertSysFile(SysFile sysFile) {
        LOGGER.info("文件资源 SYS_FILE :{}",sysFile);
        return sysFileRDao.insert(sysFile);
    }

    @Override
    @ApiOperation(value = "更新资源信息")
    public void updateSysFile(SysFile sysFile) {
        LOGGER.info("文件资源 SYS_FILE :{}",sysFile);
        sysFileRDao.update(sysFile);
    }

    @Override
    @ApiOperation(value = "查询资源信息",notes = "查询所有资源信息")
    public List<SysFile> listSysFile() {
        LOGGER.info("文件资源 SYS_FILE list");
        return sysFileRDao.list();
    }

    @Override
    @ApiOperation(value = "查询资源信息",notes = "根据 ID查询资源信息")
    public SysFile getSysFileById(Long id) {
        LOGGER.info("文件资源 SYS_FILE id:{}",id);
        return sysFileRDao.getById(id);
    }

    @Override
    @ApiOperation(value = "查询资源信息",notes = "查询资源信息(分页、组合)")
    public ListPager querySysFileByPager(SysFile sysFile, Pager pager) {
        LOGGER.info("文件资源 SYS_FILE sysFile:{}，Pager:{}",sysFile,pager);
        if(sysFile == null) {
            pager = PagerHelper.getPager(pager, sysFileRDao.listCount());
            return new ListPager(sysFileRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysFile(sysFile);
            pager = PagerHelper.getPager(pager, sysFileRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysFileRDao.findByHqlWherePage(hqlPack + sysFile.orderBy(),pager),pager);
        }
    }

    @Override
    @ApiOperation(value = "删除资源信息",notes = "根据ID删除资源信息")
    public void deleteSysFileById(Long id) {
        LOGGER.info("文件资源 SYS_FILE id:{}",id);
        sysFileRDao.deleteById(id);
    }

    @Override
    public void deleteSysFileByIds(Long[] ids) {
        LOGGER.info("文件资源 SYS_FILE ids:{}", ids.toString());
        sysFileRDao.deleteByIds(ids);
    }
}
